/*
 *  Copyright (c) 2012-2013, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information:
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *
 */

/**
 * @file        display-fb.c
 *
 * @brief       Example program to write to frame buffer
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include<stdio.h>
#include<stdlib.h>
#include<fcntl.h>
#include<unistd.h>
#include<linux/fb.h>
#include<sys/mman.h>
#include<sys/ioctl.h>

int fb, fbline_len;
unsigned long fbmem_offset, fbmem_addr;
int width, height;

/**
 * Check for root user
 * open frame buffer device and mmap it
 * set width and height to full screen
*/
int fb_init() {
    int pgSize, ret;
    struct fb_var_screeninfo vsi;
    struct fb_fix_screeninfo fsi;

    if (0 != geteuid ()) {
        printf ("Framebuffer display cannot be used without root permissions\n");
        goto error;
    }
    fb = open ("/dev/fb0", O_RDWR);
    if (fb < 0) {
        printf ("Cannot open /dev/fb0");
        goto error;
    }

    ret = ioctl (fb, FBIOGET_FSCREENINFO, &fsi);
    if (ret < 0) {
        printf ("FBIOGET_FSCREENINFO ioctl failed\n");
        goto error;
    }
    ret = ioctl (fb, FBIOGET_VSCREENINFO, &vsi);
    if (ret < 0) {
        printf ("FBIOGET_VSCREENINFO ioctl failed\n");
        goto error;
    }
    width = vsi.xres;
    height = vsi.yres;

    pgSize = sysconf (_SC_PAGESIZE);
    fbmem_offset    = (unsigned long)(fsi.smem_start) & (pgSize - 1);
    fbmem_addr      = (unsigned long int)mmap(NULL, fsi.smem_len + fbmem_offset,
                        PROT_READ | PROT_WRITE, MAP_SHARED, fb, 0);
    fbline_len      = fsi.line_length;
    if (fbmem_addr < 0) {
        printf ("mmap failed\n");
        goto error;
    }

    return 1;
error:
    return 0;
}

struct color {
    unsigned char b;
    unsigned char g;
    unsigned char r;
    unsigned char a;
};

/**
 * val is static variable  which holds 6 bits for each component
 * Next color is calculated by incrementing one of the component R,G,B
*/
void next_color(struct color *col) {
    static int val = 0444444;
    col->r = (val & 0770000) >> 10;
    col->g = (val & 0007700) >> 4;
    col->b = (val & 0000077) << 2;
    col->a = 0;
    val += 4;
}

/**
 * Copy a color structure into the mmapped memory
*/
int fb_display(struct color *col) {
    struct color *addr;
    int i,j;

    for (i=0; i<height; i++) {
        addr = (struct color *)(fbmem_addr + fbmem_offset + fbline_len * i);
        for(j=0; j<width; j++) {
            *addr = *col;
            addr++;
        }
    }
}

int main () {
    int i = 0;
    struct color col = {0,0,0,0};
    if(!fb_init()) {
            exit(1);
    }
    for (i=0; i < 500; i++) {
        next_color(&col);
        fb_display(&col);
        sleep(0.5);
    }
    col.r = 0;
    col.g = 0;
    col.b = 0;
    col.a = 0;
    fb_display(&col);
    return 0;
}
